@@ -9,14 +9,26 @@
 
 import shutil
 
+from quodlibet.util import enum
 from . import const
 from quodlibet.util.config import Config, Error
 from quodlibet.util import print_d, print_w
-from quodlibet.compat import PY2, iteritems
+from quodlibet.compat import PY2, iteritems, text_type
 
 # Some plugins can be enabled on first install
 AUTO_ENABLED_PLUGINS = ["Shuffle Playlist", "Remove Playlist Duplicates"]
 
+
+def _config_text(text):
+    # raw config values are utf-8 encoded on PY2, while they are unicode
+    # with surrogates on PY2. this makes initing the defaults work
+
+    assert isinstance(text, text_type)
+    if PY2:
+        return text.encode("utf-8")
+    return text
+
+
 # this defines the initial and default values
 INITIAL = {
     # User-defined tag name -> human name mappings
@@ -47,9 +59,16 @@ INITIAL = {
         "queue_expanded": "false", # on or off
         "shufflequeue": "false", # on or off
         "sortby": "0album", # <reversed?>tagname, song list sort
-        "order": "inorder",
-        "order_shuffle": "shuffle",
+
+        # Repeat on or off
+        "repeat": "false",
+        # The Repeat (Order) to use
+        "repeat_mode": "repeat_song",
+
+        # Shuffle on or off
         "shuffle": "false",
+        # The Shuffle (Order) to use
+        "shuffle_mode": "random",
         "plugin_selection": "", # selected plugin in manager
         "column_widths": "", # column widths in c1,w1,c2,w2... format
         "column_expands": "",
@@ -58,7 +77,7 @@ INITIAL = {
         "query_text": "", # none/search bar text
         # panes in paned browser
         "panes":
-            "~people	<~year|[b][i]<~year>[/i][/b] - ><album>",
+            "~people\t<~year|[b][i]<~year>[/i][/b] - ><album>",
         "pane_selection": "", # selected pane values
         "pane_wide_mode": "0", # browser orientation
         "background": "", # "global" filter for SearchBar
@@ -72,6 +91,9 @@ INITIAL = {
         "rating_confirm_multiple": "false", # confirm rating multiple songs
         "cover_size": "-1", # max cover height/width, <= 0 is default
         "search_limit": "false", # Show the limit widgets for SearchBar
+        "album_text": "1", # show text in covergrid view
+        "covergrid_magnification": "3.0", # show text in covergrid view
+        "covergrid_all": "0", # show "all albums" in covergrid view
     },
     # Kind of a dumping ground right now, should probably be
     # cleaned out later.
@@ -92,13 +114,10 @@ INITIAL = {
         "bayesian_rating_factor": "0.0",
 
         # rating symbol (black star)
-        "rating_symbol_full": "\xe2\x98\x85",
+        "rating_symbol_full": _config_text(u'\u2605'),
 
         # rating symbol (hollow star)
-        "rating_symbol_blank": "\xe2\x98\x86",
-
-        # probably belongs in memory
-        "repeat": "false",
+        "rating_symbol_blank": _config_text(u'\u2606'),
 
         # Now deprecated: space-separated headers column
         #"headers": " ".join(const.DEFAULT_COLUMNS),
@@ -156,6 +175,9 @@ INITIAL = {
         "prefer_embedded": "false",
         "force_filename": "false",
         "filename": "folder.jpg",
+    },
+    "display": {
+        "duration_format": "standard"
     }
 }
 
@@ -181,6 +203,8 @@ add_section = _config.add_section
 has_option = _config.has_option
 remove_option = _config.remove_option
 register_upgrade_function = _config.register_upgrade_function
+getbytes = _config.getbytes
+setbytes = _config.setbytes
 
 _filename = None
 """The filename last used for loading"""
@@ -331,3 +355,23 @@ class HardCodedRatingsPrefs(RatingsPrefs):
 
 # Need an instance just for imports to work
 RATINGS = RatingsPrefs()
+
+
+@enum
+class DurationFormat(str):
+    NUMERIC, SECONDS = "numeric", "numeric-seconds"
+    STANDARD, FULL = "standard", "text-full"
+
+
+class DurationFormatPref(object):
+    @property
+    def format(self):
+        raw = get("display", "duration_format")
+        return DurationFormat.value_of(raw, DurationFormat.STANDARD)
+
+    @format.setter
+    def format(self, value):
+        set("display", "duration_format", value)
+
+
+DURATION = DurationFormatPref()
