@@ -9,6 +9,20 @@ import sys
 import os
 
 
+# MSYS2 defines MSYSTEM which changes os.sep/os.path.sep for the mingw
+# Python build. Unset here and restart.. (does not work for py.test etc.)
+# XXX: do this here since it gets executed by all scripts
+if os.name == "nt" and "MSYSTEM" in os.environ:
+    import subprocess
+    del os.environ["MSYSTEM"]
+    argv = []
+    for arg in [sys.executable] + sys.argv:
+        if os.path.exists(arg):
+            arg = arg.replace("/", "\\")
+        argv.append(arg)
+    sys.exit(subprocess.call(argv))
+
+
 class Version(tuple):
     """Represent the version of a dependency as a tuple"""
 
@@ -47,11 +61,11 @@ class MinVersions(object):
     GSTREAMER = Version("GStreamer", 1, 0)
 
 
-VERSION_TUPLE = Version("", 3, 7, 0)
+VERSION_TUPLE = Version("", 3, 8, 0)
 VERSION = str(VERSION_TUPLE)
 
 # entry point for the user guide / wiki
-BRANCH_NAME = "quodlibet-3.7"
+BRANCH_NAME = "quodlibet-3.8"
 DOCS_BASE_URL = "https://quodlibet.readthedocs.org/en/%s"
 DOCS_LATEST = DOCS_BASE_URL % "latest"
 DOCS_BASE_URL %= BRANCH_NAME if BRANCH_NAME != "master" else "latest"
@@ -96,6 +110,7 @@ Bruno Bergot
 Carlo Teubner
 Christine Spang
 Christoph Reiter
+Corentin Néau
 David Kågedal
 David Schneider
 Decklin Foster
@@ -158,6 +173,7 @@ Zack Weinberg
 """.strip().split("\n"))
 
 TRANSLATORS = sorted(u"""
+Åka Sikrom (nb)
 Alexandre Passos (pt)
 Andreas Bertheussen (nb)
 Anton Shestakov (ru)
@@ -210,6 +226,8 @@ Yasushi Iwata (ja)
 Николай Прокошенко (ru)
 Ростислав "zbrox" Райков (bg)
 Сергей Федосеев (ru)
+scootergrisen (da)
+Marek Suchánek (cs)
 """.strip().splitlines())
 
 ARTISTS = sorted(u"""\
