@@ -8,8 +8,11 @@
 import re
 import shlex
 
+from senf import bytes2fsn, fsn2bytes
+
 from quodlibet import const
 from quodlibet.util import print_d, print_w
+from quodlibet.compat import text_type, iteritems
 from .tcpserver import BaseTCPServer, BaseTCPConnection
 
 
@@ -98,18 +101,18 @@ def parse_command(line):
 
     assert isinstance(line, bytes)
 
-    parts = re.split("[ \\t]+", line, maxsplit=1)
+    parts = re.split(b"[ \\t]+", line, maxsplit=1)
     if not parts:
         raise ParseError("empty command")
     command = parts[0]
 
     if len(parts) > 1:
-        lex = shlex.shlex(parts[1], posix=True)
+        lex = shlex.shlex(bytes2fsn(parts[1], "utf-8"), posix=True)
         lex.whitespace_split = True
         lex.commenters = ""
         lex.quotes = "\""
         lex.whitespace = " \t"
-        args = list(lex)
+        args = [fsn2bytes(a, "utf-8") for a in lex]
     else:
         args = []
 
@@ -152,7 +155,7 @@ class MPDService(object):
         def options_changed(*args):
             self.emit_changed("options")
 
-        self._options.connect("notify::random", options_changed)
+        self._options.connect("notify::shuffle", options_changed)
         self._options.connect("notify::repeat", options_changed)
         self._options.connect("notify::single", options_changed)
 
@@ -208,7 +211,7 @@ class MPDService(object):
 
     def flush_idle(self):
         flushed = []
-        for conn, subs in self._idle_subscriptions.iteritems():
+        for conn, subs in iteritems(self._idle_subscriptions):
             # figure out which subsystems to report for each connection
             queued = self._idle_queue[conn]
             if subs:
@@ -233,7 +236,7 @@ class MPDService(object):
         self._idle_subscriptions.pop(connection, None)
 
     def emit_changed(self, subsystem):
-        for conn, subs in self._idle_queue.iteritems():
+        for conn, subs in iteritems(self._idle_queue):
             subs.add(subsystem)
         self.flush_idle()
 
@@ -287,7 +290,7 @@ class MPDService(object):
         self._options.repeat = value
 
     def random(self, value):
-        self._options.random = value
+        self._options.shuffle = value
 
     def single(self, value):
         self._options.single = value
@@ -321,7 +324,7 @@ class MPDService(object):
         status = [
             ("volume", int((app.player.volume ** (1.0 / 3.0)) * 100)),
             ("repeat", int(self._options.repeat)),
-            ("random", int(self._options.random)),
+            ("random", int(self._options.shuffle)),
             ("single", int(self._options.single)),
             ("consume", 0),
             ("playlist", self._pl_ver),
@@ -421,8 +424,8 @@ class MPDConnection(BaseTCPConnection):
         self.service = service
         service.add_connection(self)
 
-        str_version = ".".join(map(str, service.version))
-        self._buf = bytearray("OK MPD %s\n" % str_version)
+        str_version = u".".join(map(text_type, service.version))
+        self._buf = bytearray((u"OK MPD %s\n" % str_version).encode("utf-8"))
         self._read_buf = bytearray()
 
         # begin - command processing state
@@ -496,7 +499,7 @@ class MPDConnection(BaseTCPConnection):
         """Returns the next line from the read buffer or None"""
 
         try:
-            index = self._read_buf.index("\n")
+            index = self._read_buf.index(b"\n")
         except ValueError:
             return None
 
@@ -507,10 +510,10 @@ class MPDConnection(BaseTCPConnection):
     def write_line(self, line):
         """Writes a line to the client"""
 
-        assert isinstance(line, unicode)
+        assert isinstance(line, text_type)
         self.log(u"<- " + repr(line))
 
-        self._buf.extend(line.encode("utf-8", errors="replace") + "\n")
+        self._buf.extend(line.encode("utf-8", errors="replace") + b"\n")
 
     def ok(self):
         self.write_line(u"OK")
@@ -837,7 +840,7 @@ def _cmd_outputs(conn, service, args):
 @MPDConnection.Command("commands", permission=Permissions.PERMISSION_NONE)
 def _cmd_commands(conn, service, args):
     for name in conn.list_commands():
-        conn.write_line(u"command: " + unicode(name))
+        conn.write_line(u"command: " + text_type(name))
 
 
 @MPDConnection.Command("tagtypes")
