@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 # Copyright 2004-2009 Joe Wreschnig, Michael Urman, Steven Robertson
-#           2011,2013 Nick Boultbee
+#      2011,2013,2016 Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -8,6 +8,9 @@
 
 from gi.repository import GObject, Gtk
 
+from quodlibet.browsers.playlists.menu import PlaylistMenu
+
+from quodlibet import _
 from quodlibet import browsers
 from quodlibet import qltk
 from quodlibet.qltk.ratingsmenu import RatingsMenuItem
@@ -73,9 +76,9 @@ class IndicatorMenu(Gtk.Menu):
         player_options = app.player_options
 
         shuffle = Gtk.CheckMenuItem(label=_("_Shuffle"), use_underline=True)
-        player_options.bind_property("random", shuffle, "active",
+        player_options.bind_property("shuffle", shuffle, "active",
                                      GObject.BindingFlags.BIDIRECTIONAL)
-        player_options.notify("random")
+        player_options.notify("shuffle")
 
         repeat = Gtk.CheckMenuItem(label=_("_Repeat"), use_underline=True)
         player_options.bind_property("repeat", repeat, "active",
@@ -92,8 +95,6 @@ class IndicatorMenu(Gtk.Menu):
         browse_sub = Gtk.Menu()
 
         for Kind in browsers.browsers:
-            if Kind.is_empty:
-                continue
             i = Gtk.MenuItem(label=Kind.accelerated_name, use_underline=True)
             connect_obj(i,
                 'activate', LibraryBrowser.open, Kind, app.library, app.player)
@@ -101,7 +102,7 @@ class IndicatorMenu(Gtk.Menu):
 
         browse.set_submenu(browse_sub)
 
-        self._props = qltk.MenuItem(_("Edit _Tags"), Icons.DOCUMENT_PROPERTIES)
+        self._props = qltk.MenuItem(_("Edit _Tags"), Icons.EDIT)
 
         def on_properties(*args):
             song = player.song
@@ -112,6 +113,10 @@ class IndicatorMenu(Gtk.Menu):
 
         self._info = MenuItem(_("_Information"), Icons.DIALOG_INFORMATION)
 
+        self._playlists_item = MenuItem(_("Play_lists"),
+                                        Icons.FOLDER_DRAG_ACCEPT)
+        self._new_playlist_submenu_for(player.song)
+
         def on_information(*args):
             song = player.song
             window = Information(app.librarian, [song])
@@ -143,6 +148,7 @@ class IndicatorMenu(Gtk.Menu):
         self.append(safter)
         self.append(SeparatorMenuItem())
         self.append(rating)
+        self.append(self._playlists_item)
         self.append(self._props)
         self.append(self._info)
         self.append(SeparatorMenuItem())
@@ -184,6 +190,16 @@ class IndicatorMenu(Gtk.Menu):
         self._info.set_sensitive(song is not None)
         self._props.set_sensitive(song is not None)
         self._rating_item.set_songs([song])
+        self._new_playlist_submenu_for(song)
+
+    def _new_playlist_submenu_for(self, song):
+        submenu = self._playlists_item.get_submenu()
+        if submenu:
+            submenu.destroy()
+        playlist_menu = PlaylistMenu([song])
+        self._playlists_item.set_submenu(playlist_menu)
+        self._playlists_item.set_sensitive(bool(song) and song.can_add)
+        self._playlists_item.show_all()
 
     def _on_play_pause(self, menuitem, player):
         if player.song:
