@@ -19,6 +19,7 @@ from quodlibet import const
 from quodlibet import config
 from quodlibet import util
 from quodlibet import app
+from quodlibet import _
 
 from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
     MainRunnerAbortedError, MainRunnerTimeoutError, print_w, print_d, \
@@ -26,6 +27,7 @@ from quodlibet.util import fver, sanitize_tags, MainRunner, MainRunnerError, \
 from quodlibet.player import PlayerError
 from quodlibet.player._base import BasePlayer
 from quodlibet.qltk.notif import Task
+from quodlibet.compat import text_type, iteritems
 
 from .util import (parse_gstreamer_taglist, TagListWrapper, iter_to_list,
     GStreamerSink, link_many, bin_debug)
@@ -409,7 +411,8 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
                         "mpg123audiodec": -2
                     }.get(f.get_name(), i)
                     return (i, f)
-                return zip(*sorted(map(set_prio, enumerate(factories))))[1]
+                return list(
+                    zip(*sorted(map(set_prio, enumerate(factories)))))[1]
 
             for e in iter_to_list(self.bin.iterate_recurse):
                 try:
@@ -774,7 +777,7 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         self.error = True
         self.paused = True
 
-        print_w(unicode(player_error))
+        print_w(text_type(player_error))
         self.emit('error', self.song, player_error)
         self._active_error = False
 
@@ -879,12 +882,12 @@ class GStreamerPlayer(BasePlayer, GStreamerPluginHandler):
         tags = TagListWrapper(tags, merge=True)
         tags = parse_gstreamer_taglist(tags)
 
-        for key, value in sanitize_tags(tags, stream=False).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=False)):
             if self.song.get(key) != value:
                 changed = True
                 self.song[key] = value
 
-        for key, value in sanitize_tags(tags, stream=True).iteritems():
+        for key, value in iteritems(sanitize_tags(tags, stream=True)):
             if new_info.get(key) != value:
                 info_changed = True
                 new_info[key] = value
