@@ -8,7 +8,6 @@
 # published by the Free Software Foundation
 
 import os
-import sys
 import signal
 
 import gi
@@ -16,11 +15,41 @@ gi.require_version("Gtk", "3.0")
 
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GLib
+from gi.repository import GLib, GObject
+from senf import fsn2bytes, bytes2fsn
 
 from quodlibet.util import gdecode, print_d, print_w
 
 
+def get_fg_highlight_color(widget):
+    """Returns a color useable for highlighting things on top of the standard
+    background color.
+
+    Args:
+        widget (Gtk.Widget)
+    Returns:
+        Gdk.RGBA
+    """
+
+    context = widget.get_style_context()
+    if hasattr(Gtk.StateFlags, "LINK"):
+        # gtk+ >=3.12
+        context.save()
+        context.set_state(Gtk.StateFlags.LINK)
+        color = context.get_color(context.get_state())
+        context.restore()
+    else:
+        value = GObject.Value()
+        value.init(Gdk.Color)
+        value.set_boxed(None)
+        context.get_style_property("link-color", value)
+        color = Gdk.RGBA()
+        old_color = value.get_boxed()
+        if old_color is not None:
+            color.parse(old_color.to_string())
+    return color
+
+
 def get_primary_accel_mod():
     """Returns the primary Gdk.ModifierType modifier.
 
@@ -49,13 +78,9 @@ def selection_set_songs(selection_data, songs):
 
     filenames = []
     for filename in (song["~filename"] for song in songs):
-        if isinstance(filename, unicode):
-            # win32
-            filename = filename.encode("utf-8")
-        filenames.append(filename)
-
+        filenames.append(fsn2bytes(filename, "utf-8"))
     type_ = Gdk.atom_intern("text/x-quodlibet-songs", True)
-    selection_data.set(type_, 8, "\x00".join(filenames))
+    selection_data.set(type_, 8, b"\x00".join(filenames))
 
 
 def selection_get_filenames(selection_data):
@@ -66,11 +91,8 @@ def selection_get_filenames(selection_data):
     data_type = selection_data.get_data_type()
     assert data_type.name() == "text/x-quodlibet-songs"
 
-    items = selection_data.get_data().split("\x00")
-    if sys.platform == "win32":
-        return [item.decode("utf-8") for item in items]
-    else:
-        return items
+    items = selection_data.get_data().split(b"\x00")
+    return [bytes2fsn(i, "utf-8") for i in items]
 
 
 def get_top_parent(widget):
@@ -270,6 +292,11 @@ def add_css(widget, css):
     context.add_provider(provider, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION)
 
 
+def remove_padding(widget):
+    """Removes padding on supplied widget"""
+    return add_css(widget, " * { padding: 0px; } ")
+
+
 def is_wayland():
     # FIXME: Is there no better way?
     display = Gdk.Display.get_default()
@@ -401,6 +428,53 @@ def add_signal_watch(signal_action):
         unix_signal_add(GLib.PRIORITY_HIGH, signum, handler, signum)
 
 
+class ThemeOverrider(object):
+    """Allows registering global Gtk.StyleProviders for a specific theme.
+    They get activated when the theme gets active and removed when the theme
+    changes to something else.
+    """
+
+    def __init__(self):
+        self._providers = {}
+        self._active_providers = []
+        settings = Gtk.Settings.get_default()
+        settings.connect("notify::gtk-theme-name", self._on_theme_name_notify)
+        settings.notify("gtk-theme-name")
+
+    def register_provider(self, theme_name, provider):
+        """
+        Args:
+            theme_name (str): A gtk+ theme name e.g. "Adwaita" or empty to
+                apply to all themes
+            provider (Gtk.StyleProvider)
+        """
+
+        self._providers.setdefault(theme_name, []).append(provider)
+        settings = Gtk.Settings.get_default()
+        settings.notify("gtk-theme-name")
+
+    def _on_theme_name_notify(self, settings, gparam):
+
+        theme_name = settings.get_property(gparam.name)
+        wanted_providers = \
+            self._providers.get(theme_name, []) + self._providers.get("", [])
+
+        for provider in list(self._active_providers):
+            if provider not in wanted_providers:
+                Gtk.StyleContext.remove_provider_for_screen(
+                    Gdk.Screen.get_default(), provider)
+            self._active_providers.remove(provider)
+
+        for provider in wanted_providers:
+            if provider not in self._active_providers:
+                Gtk.StyleContext.add_provider_for_screen(
+                    Gdk.Screen.get_default(),
+                    provider,
+                    Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION
+                )
+                self._active_providers.append(provider)
+
+
 from .msg import Message, ErrorMessage, WarningMessage
 from .x import Align, Button, ToggleButton, Notebook, SeparatorMenuItem, \
     WebImage, MenuItem, Frame, EntryCompletion
