@@ -10,6 +10,7 @@ from gi.repository import Gtk
 
 from quodlibet.qltk.pluginwin import PluginWindow
 
+from quodlibet import ngettext, _
 from quodlibet import qltk
 
 from quodlibet.util import print_exc, print_e, print_w
@@ -23,6 +24,7 @@ from quodlibet.qltk import get_top_parent, get_menu_item_top_parent, Icons
 from quodlibet.plugins import PluginManager, PluginHandler
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
 from quodlibet.util.songwrapper import ListWrapper, check_wrapper_changed
+from quodlibet.compat import listfilter
 
 
 class ConfirmMultiSongInvoke(WarningMessage):
@@ -117,7 +119,7 @@ class SongsMenuPluginHandler(PluginHandler):
         kinds = self.__plugins
         kinds.sort(key=lambda plugin: plugin.PLUGIN_ID)
         for Kind in kinds:
-            usable = max([callable(getattr(Kind, s)) for s in attrs])
+            usable = any(callable(getattr(Kind, s)) for s in attrs)
             if usable:
                 try:
                     items.append(Kind(songs, library))
@@ -125,7 +127,7 @@ class SongsMenuPluginHandler(PluginHandler):
                     print_e("Couldn't initialise song plugin %s. Stack trace:"
                             % Kind)
                     print_exc()
-        items = filter(lambda i: i.initialized, items)
+        items = listfilter(lambda i: i.initialized, items)
 
         if items:
             menu = Gtk.Menu()
@@ -210,7 +212,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 except Exception:
                     print_exc()
                 else:
-                    if max(ret):
+                    if any(ret):
                         return
             if callable(plugin.plugin_songs):
                 try:
@@ -243,7 +245,7 @@ class SongsMenuPluginHandler(PluginHandler):
                 except Exception:
                     print_exc()
                 else:
-                    if max(ret):
+                    if any(ret):
                         return
             if callable(plugin.plugin_albums):
                 try:
@@ -329,7 +331,7 @@ class SongsMenu(Gtk.Menu):
             except AttributeError as e:
                 print_w("Couldn't get Playlists menu: %s" % e)
             else:
-                b = qltk.MenuItem(_("Play_lists"), Icons.LIST_ADD)
+                b = qltk.MenuItem(_("Play_lists"), Icons.FOLDER_DRAG_ACCEPT)
                 b.set_sensitive(can_add and bool(songs))
                 b.set_submenu(submenu)
                 self.append(b)
@@ -357,11 +359,12 @@ class SongsMenu(Gtk.Menu):
             else:
                 if browsers.media.MediaDevices in browsers.browsers:
                     submenu = browsers.media.Menu(songs, library)
-                    b = qltk.MenuItem(_("_Copy to Device"), Icons.EDIT_COPY)
-                    b.set_sensitive(
-                        can_add and len(submenu) > 0 and bool(songs))
-                    b.set_submenu(submenu)
-                    self.append(b)
+                    b = qltk.MenuItem(_("_Copy to Device"),
+                                      Icons.MULTIMEDIA_PLAYER)
+                    if can_add and len(submenu) > 0 and bool(songs):
+                        b.set_sensitive(True)
+                        b.set_submenu(submenu)
+                        self.append(b)
 
         if remove or delete:
             self.separate()
@@ -399,7 +402,7 @@ class SongsMenu(Gtk.Menu):
 
         if edit:
             self.separate()
-            b = qltk.MenuItem(_("Edit _Tags"), Icons.DOCUMENT_PROPERTIES)
+            b = qltk.MenuItem(_("Edit _Tags"), Icons.EDIT)
             b.set_sensitive(bool(songs))
             if accels:
                 qltk.add_fake_accel(b, "<alt>Return")
